@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
        
        @author Azzam Haidar
        @author Ahmad Abdelfattah
@@ -16,7 +16,7 @@ extern "C" {
 #endif
 
 // Definition of blocking sizes for NVIDIA cards
-#ifdef HAVE_CUBLAS
+#if defined(MAGMA_HAVE_CUDA) || defined(MAGMA_HAVE_HIP)
 
 // =============================================================================
 /// @addtogroup magma_tuning
@@ -525,7 +525,7 @@ magma_int_t magma_zrecommend_cublas_gemm_stream(
 /// @}
 // end group magma_tuning
 
-#endif  // HAVE_CUBLAS
+#endif  // MAGMA_HAVE_CUDA
 
 #ifdef __cplusplus
 } // extern "C"
