@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Stan Tomov
        @author Mark Gates
@@ -16,8 +16,9 @@
 extern "C" {
 #endif
 
+// TODO: also add some for AMD cards
 // Definition of blocking sizes for NVIDIA cards
-#ifdef HAVE_CUBLAS
+//#ifdef MAGMA_HAVE_CUDA
 
 // =============================================================================
 /// @addtogroup magma_tuning
@@ -55,7 +56,8 @@ magma_int_t magma_get_dpotrf_nb( magma_int_t n )
     magma_int_t nb;
     magma_int_t arch = magma_getdevice_arch();
     if ( arch >= 300 ) {       // 3.x Kepler
-        if      (n <  3072) nb = 256;
+        if      (n <  2048) nb = 128;
+        else if (n <  3072) nb = 256;
         else                nb = 512;
     }
     else if ( arch >= 200 ) {  // 2.x Fermi
@@ -165,7 +167,10 @@ magma_int_t magma_get_sgeqrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {
         if      (minmn <  4096) nb = 96;
         else if (minmn <  7168) nb = 128;
         else if (minmn < 18432) nb = 256;
@@ -190,8 +195,12 @@ magma_int_t magma_get_dgeqrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
-        if      (minmn <  3072) nb = 64;
+        if ( arch >= 800 ) {
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
+        if      (minmn <   256) nb =  64;
+        else if (minmn <  3072) nb = 128;
         else if (minmn < 10240) nb = 128;
         else                    nb = 256;
     }
@@ -208,7 +217,10 @@ magma_int_t magma_get_cgeqrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+        if ( arch >= 800 ) {
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn <  4096) nb = 64;
         else                    nb = 128;
     }
@@ -226,7 +238,10 @@ magma_int_t magma_get_zgeqrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+        if ( arch >= 800 ) {
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn <  4096) nb = 64;
         else                    nb = 128;
     }
@@ -630,7 +645,13 @@ magma_int_t magma_get_sgetrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        // high-end GPUs are much faster thanmost CPUs, so small nb's usually
+        // gives a higher performance than bigger nb's
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {
         if      (minmn <  4096) nb = 256;
         else if (minmn < 18432) nb = 512;
         else                    nb = 1024;
@@ -653,8 +674,15 @@ magma_int_t magma_get_dgetrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
-        if      (minmn <  3072) nb = 128;
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        if      (minmn <=  9000) nb = 32;
+        else if (minmn <= 14000) nb = 64;
+        else                     nb = 96;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
+        if      (minmn <  2048) nb =  64;
+        else if (minmn <  3072) nb = 128;
         else if (minmn <  8192) nb = 256;
         else                    nb = 512;
     }
@@ -676,7 +704,11 @@ magma_int_t magma_get_cgetrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn < 4096) nb = 64;
         else if (minmn < 8192) nb = 256;
         else                   nb = 512;
@@ -698,7 +730,11 @@ magma_int_t magma_get_zgetrf_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        nb = 32;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn < 4096) nb = 64;
         else if (minmn < 8192) nb = 256;
         else                   nb = 512;
@@ -721,7 +757,11 @@ magma_int_t magma_get_sgetrf_native_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        nb = 512;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn <=  4096) nb = 64;
         else if (minmn <= 10240) nb = 128;
         else if (minmn <= 20480) nb = 512;
@@ -745,7 +785,12 @@ magma_int_t magma_get_dgetrf_native_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        if ( minmn <= 7000 ) nb = 256;
+        else                 nb = 512;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn <=  4096)  nb = 64;
         else if (minmn <=  10240) nb = 128;
         else if (minmn <=  20480) nb = 512;
@@ -769,8 +814,12 @@ magma_int_t magma_get_cgetrf_native_nb( magma_int_t m, magma_int_t n )
     magma_int_t nb;
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
-    // TODO: try all nb's (128,256,512, ... etc)
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        if ( minmn <= 15000 ) nb = 64;
+        else                  nb = 512;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn <=  2048)  nb = 128;
         else                      nb = 256;
     }
@@ -792,7 +841,12 @@ magma_int_t magma_get_zgetrf_native_nb( magma_int_t m, magma_int_t n )
     magma_int_t minmn = min( m, n );
     magma_int_t arch = magma_getdevice_arch();
     // TODO: try all nb's (128,256,512, ... etc)
-    if ( arch >= 300 ) {       // 3.x Kepler
+    if ( arch >= 800 ) {
+        // based on experiments using AMD CPU running MKL + A100-SXM4 GPU
+        if ( minmn <= 8000 ) nb = 256;
+        else                 nb = 512;
+    }
+    else if ( arch >= 300 ) {       // 3.x Kepler
         if      (minmn <=  4096)  nb = 32;
         else if (minmn <=  10240) nb = 64;
         else if (minmn <=  20480) nb = 256;
@@ -1498,7 +1552,7 @@ magma_int_t magma_get_zbulge_mgpu_nb( magma_int_t n )
 /// @}
 // end group magma_tuning
 
-#endif  // HAVE_CUBLAS
+//#endif  // MAGMA_HAVE_CUDA
 
 #ifdef __cplusplus
 } // extern "C"
