@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 */
 
 #ifndef MAGMA_H
@@ -13,10 +13,17 @@
 #error "Since MAGMA_NO_V1 is defined, magma.h is invalid; use magma_v2.h"
 #endif
 
+// =============================================================================
+// MAGMA configuration
+#include "magma_config.h"
+
+
 // magma v1 includes cublas.h by default, unless cublas_v2.h has already been included
 #ifndef CUBLAS_V2_H_
+#if defined(MAGMA_HAVE_CUDA)
 #include <cublas.h>
 #endif
+#endif
 
 // Include the MAGMA v2 and v1 APIs,
 // then map names to the v1 API (e.g., magma_zgemm => magma_zgemm_v1).
