@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 */
 
 #ifndef MAGMA_V2_H
@@ -12,6 +12,10 @@
 #define MAGMA_API 2
 
 
+// =============================================================================
+// MAGMA configuration
+#include "magma_config.h"
+
 // =============================================================================
 // MAGMA BLAS Functions
 
