@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @precisions normal z -> s d c
 */
@@ -75,6 +75,7 @@ magma_int_t magma_get_zbulge_gcperf();
 // only applicable to real [sd] precisions
 magma_int_t
 magma_dsidi(
+    magma_uplo_t uplo,
     double *A, magma_int_t lda, magma_int_t n, magma_int_t *ipiv,
     double *det, magma_int_t *inert,
     double *work, magma_int_t job,
@@ -339,6 +340,17 @@ magma_zgeqrf_gpu(
     magmaDoubleComplex_ptr dT,
     magma_int_t *info);
 
+magma_int_t
+magma_zgeqrf_expert_gpu_work(
+    magma_int_t m, magma_int_t n,
+    magmaDoubleComplex_ptr dA, magma_int_t ldda,
+    magmaDoubleComplex *tau, magmaDoubleComplex_ptr dT,
+    magma_int_t *info,
+    magma_mode_t mode, magma_int_t nb,
+    void* host_work,   magma_int_t *lwork_host,
+    void* device_work, magma_int_t *lwork_device,
+    magma_queue_t queues[2] );
+
 // CUDA MAGMA only
 magma_int_t
 magma_zgeqrf_m(
@@ -489,20 +501,21 @@ magma_zgetf2_gpu(
     magma_queue_t queue,
     magma_int_t *info);
 
-magma_int_t 
-magma_zgetf2_native_fused( 
-    magma_int_t m, magma_int_t n, 
-    magmaDoubleComplex_ptr dA, magma_int_t ldda, 
-    magma_int_t *ipiv, magma_int_t gbstep, 
-    magma_int_t *flags, 
+magma_int_t
+magma_zgetf2_native_fused(
+    magma_int_t m, magma_int_t n,
+    magmaDoubleComplex_ptr dA, magma_int_t ldda,
+    magma_int_t *ipiv, magma_int_t gbstep,
+    magma_int_t *flags,
     magma_int_t *info, magma_queue_t queue );
 
 magma_int_t
 magma_zgetf2_native(
     magma_int_t m, magma_int_t n,
     magmaDoubleComplex_ptr dA, magma_int_t ldda,
-    magma_int_t *dipiv, magma_int_t* dipivinfo, 
-    magma_int_t *dinfo, magma_int_t gbstep, 
+    magma_int_t *dipiv, magma_int_t* dipivinfo,
+    magma_int_t *dinfo, magma_int_t gbstep,
+    magma_event_t events[2],
     magma_queue_t queue, magma_queue_t update_queue);
 
 // CUDA MAGMA only
@@ -514,11 +527,11 @@ magma_zgetf2_nopiv(
 
 magma_int_t
 magma_zgetrf_recpanel_native(
-    magma_int_t m, magma_int_t n,    
+    magma_int_t m, magma_int_t n, magma_int_t recnb,
     magmaDoubleComplex_ptr dA, magma_int_t ldda,
     magma_int_t* dipiv, magma_int_t* dipivinfo,
-    magma_int_t *dinfo, magma_int_t gbstep, 
-    magma_queue_t queue, magma_queue_t update_queue );
+    magma_int_t *dinfo, magma_int_t gbstep,
+    magma_event_t events[2], magma_queue_t queue, magma_queue_t update_queue);
 
 magma_int_t
 magma_zgetrf(
@@ -534,6 +547,25 @@ magma_zgetrf_gpu(
     magma_int_t *ipiv,
     magma_int_t *info);
 
+magma_int_t
+magma_zgetrf_expert_gpu_work(
+    magma_int_t m, magma_int_t n,
+    magmaDoubleComplex_ptr dA, magma_int_t ldda,
+    magma_int_t *ipiv,
+    magma_int_t *info, magma_mode_t mode,
+    magma_int_t nb, magma_int_t recnb,
+    void* host_work,   magma_int_t *lwork_host,
+    void* device_work, magma_int_t *lwork_device,
+    magma_event_t events[2], magma_queue_t queues[2] );
+
+magma_int_t
+magma_zgetrf_gpu_expert(
+    magma_int_t m, magma_int_t n,
+    magmaDoubleComplex_ptr dA, magma_int_t ldda,
+    magma_int_t *ipiv,
+    magma_int_t *info,
+    magma_int_t nb, magma_mode_t mode);
+
 magma_int_t
 magma_zgetrf_native(
     magma_int_t m, magma_int_t n,
@@ -1050,7 +1082,7 @@ magma_int_t
 magma_zhetrf_gpu(
    magma_uplo_t uplo, magma_int_t n,
    magmaDoubleComplex *dA, magma_int_t ldda,
-   magma_int_t *ipiv, 
+   magma_int_t *ipiv,
    magma_int_t *info);
 
 // CUDA MAGMA only
@@ -1391,8 +1423,8 @@ magma_zpotf2_gpu(
 
 magma_int_t
 magma_zpotrf_rectile_native(
-    magma_uplo_t uplo, magma_int_t n, magma_int_t recnb,    
-    magmaDoubleComplex* dA,    magma_int_t ldda, magma_int_t gbstep, 
+    magma_uplo_t uplo, magma_int_t n, magma_int_t recnb,
+    magmaDoubleComplex* dA,    magma_int_t ldda, magma_int_t gbstep,
     magma_int_t *dinfo,  magma_int_t *info, magma_queue_t queue);
 
 magma_int_t
@@ -1401,6 +1433,24 @@ magma_zpotrf(
     magmaDoubleComplex *A, magma_int_t lda,
     magma_int_t *info);
 
+magma_int_t
+magma_zpotrf_expert_gpu_work(
+    magma_uplo_t uplo, magma_int_t n,
+    magmaDoubleComplex_ptr dA, magma_int_t ldda,
+    magma_int_t *info,
+    magma_mode_t mode,
+    magma_int_t nb, magma_int_t recnb,
+    void* host_work,   magma_int_t *lwork_host,
+    void* device_work, magma_int_t *lwork_device,
+    magma_event_t events[2], magma_queue_t queues[2] );
+
+magma_int_t
+magma_zpotrf_expert_gpu(
+    magma_uplo_t uplo, magma_int_t n,
+    magmaDoubleComplex_ptr dA, magma_int_t ldda,
+    magma_int_t *info,
+    magma_int_t nb, magma_mode_t mode );
+
 magma_int_t
 magma_zpotrf_gpu(
     magma_uplo_t uplo, magma_int_t n,
@@ -1818,6 +1868,30 @@ void magma_zq_to_panel(
     magmaDoubleComplex *A, magma_int_t lda,
     magmaDoubleComplex *work);
 
+/* auxiliary routines for posv-irgmres  */
+void
+magmablas_zextract_diag_sqrt(
+    magma_int_t m, magma_int_t n,
+    magmaDoubleComplex* dA, magma_int_t ldda,
+    double* dD, magma_int_t incd,
+    magma_queue_t queue);
+
+void
+magmablas_zscal_shift_hpd(
+    magma_uplo_t uplo, int n,
+    magmaDoubleComplex* dA, int ldda,
+    double* dD, int incd,
+    double miu, double cn, double eps,
+    magma_queue_t queue);
+
+void
+magmablas_zdimv_invert(
+    magma_int_t n,
+    magmaDoubleComplex alpha, magmaDoubleComplex* dD, magma_int_t incd,
+                              magmaDoubleComplex* dx, magma_int_t incx,
+    magmaDoubleComplex beta,  magmaDoubleComplex* dy, magma_int_t incy,
+    magma_queue_t queue);
+
 #ifdef __cplusplus
 }
 #endif
