@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Hartwig Anzt
 
@@ -12,17 +12,33 @@
 #include "magmasparse_internal.h"
 #include <cuda.h>  // for CUDA_VERSION
 
+#include "magma_trisolve.h"
+
 #define PRECISION_z
 
+/* For hipSPARSE, they use a separate complex type than for hipBLAS */
+#if defined(MAGMA_HAVE_HIP)
+  #ifdef PRECISION_z
+    #define hipblasDoubleComplex hipDoubleComplex
+  #elif defined(PRECISION_c)
+    #define hipblasComplex hipComplex
+  #endif
+#endif
+
+#if CUDA_VERSION >= 12000
+   #define cusparseCreateCsrsm2Info(info)
+   #define cusparseDestroyCsrsm2Info(info)
+#endif
+
 // todo: make it spacific
-#if CUDA_VERSION >= 11000 
+#if CUDA_VERSION >= 11000 || defined(MAGMA_HAVE_HIP)
 #define cusparseCreateSolveAnalysisInfo(info) cusparseCreateCsrsm2Info(info) 
 #else
-#define cusparseCreateSolveAnalysisInfo(info)                                                   \ 
+#define cusparseCreateSolveAnalysisInfo(info)                                                   \
         CHECK_CUSPARSE( cusparseCreateSolveAnalysisInfo( info ))
 #endif
 
-#if CUDA_VERSION >= 11000
+#if CUDA_VERSION >= 11000 || defined(MAGMA_HAVE_HIP)
 #define cusparseDestroySolveAnalysisInfo(info) cusparseDestroyCsrsm2Info(info)
 #endif
 
@@ -34,6 +50,7 @@
         cuDoubleComplex *B;                                                                     \
         size_t bufsize;                                                                         \
         void *buf;                                                                              \
+        cusparseSetMatType( descrA, CUSPARSE_MATRIX_TYPE_GENERAL );                             \
         cusparseZcsrsm2_bufferSizeExt(handle, 0, op, CUSPARSE_OPERATION_NON_TRANSPOSE,          \
                                       rows, 1, nnz, (const cuDoubleComplex *)&alpha,            \
                                       descrA, dval, drow, dcol,                                 \
@@ -47,6 +64,30 @@
         if (bufsize > 0)                                                                        \
            magma_free(buf);                                                                     \
     }
+
+#elif defined(MAGMA_HAVE_HIP)
+#define cusparseZcsrsm_analysis(handle, op, rows, nnz, descrA, dval, drow, dcol, info )         \
+    {                                                                                           \
+        magmaDoubleComplex alpha = MAGMA_Z_ONE;                                                 \
+        hipDoubleComplex *B;                                                                     \
+        size_t bufsize;                                                                         \
+        void *buf;                                                                              \
+        hipsparseZcsrsm2_bufferSizeExt(handle, 0, op, HIPSPARSE_OPERATION_NON_TRANSPOSE,         \
+                                      rows, 1, nnz, (const hipDoubleComplex *)&alpha,           \
+                                      descrA, (const hipDoubleComplex *)dval, (const int *)drow, (const int *)dcol,  \
+                                      (const hipDoubleComplex *)B, rows, info, HIPSPARSE_SOLVE_POLICY_NO_LEVEL, &bufsize); \
+        if (bufsize > 0)                                                                        \
+           magma_malloc(&buf, bufsize);                                                         \
+        hipsparseZcsrsm2_analysis(handle, 0, op, HIPSPARSE_OPERATION_NON_TRANSPOSE,               \
+                                 rows, 1, nnz, (const hipDoubleComplex *)&alpha,                 \
+                                 descrA, (const hipDoubleComplex *)dval, drow, dcol,            \
+                                 B, rows, info, HIPSPARSE_SOLVE_POLICY_NO_LEVEL, buf);           \
+        if (bufsize > 0)                                                                        \
+           magma_free(buf);                                                                     \
+    }
+
+
+
 #endif
 
 #if CUDA_VERSION >= 11000
@@ -67,30 +108,6 @@
     }
 #endif
 
-
-#if CUDA_VERSION >= 11000
-#define cusparseZcsrsm_solve(handle, op, rows, cols, nnz, alpha, descrA, dval, drow, dcol,      \
-                             info, b, ldb, x, ldx )                                             \
-    {                                                                                           \
-        size_t bufsize;                                                                         \
-        void *buf;                                                                              \
-        cusparseZcsrsm2_bufferSizeExt(handle, 0, op, CUSPARSE_OPERATION_NON_TRANSPOSE,          \
-                                      rows, cols, nnz, alpha, descrA, dval, drow, dcol,         \
-                                      b, ldb, info, CUSPARSE_SOLVE_POLICY_NO_LEVEL, &bufsize);  \ 
-        magma_malloc(&buf, bufsize);                                                            \
-        cusparseZcsrsm2_solve(handle, 0, op, CUSPARSE_OPERATION_NON_TRANSPOSE, rows, cols, nnz, \
-                              alpha, descrA, dval, drow, dcol, b, ldb, info,                    \
-                              CUSPARSE_SOLVE_POLICY_NO_LEVEL, buf);                             \
-        magmablas_zlacpy( MagmaFull, rows, cols, b, ldb, x, ldx, queue );                       \
-        magma_free(buf);                                                                        \
-    }  
-#else
-#define cusparseZcsrsm_solve(handle, op, rows, cols, nnz, alpha, descrA, dval, drow, dcol,      \
-                             info, b, ldb, x, ldx )                                             \
-    CHECK_CUSPARSE( cusparseZcsrsm_solve(handle, op, rows, cols, alpha, descrA, dval,           \
-                                         drow, dcol, info, b, ldb, x, ldx ))              
-#endif 
-
 // todo: info is passed from analysis; to change info with this linfo & remove linfo from here
 #if CUDA_VERSION >= 11000
 #define cusparseZcsric0(handle, op, rows, nnz, descrA, dval, drow, dcol, info )                 \
@@ -112,6 +129,27 @@
                          CUSPARSE_SOLVE_POLICY_NO_LEVEL, buf);                                  \
         if (bufsize > 0)                                                                        \
            magma_free(buf);                                                                     \
+    }
+#elif defined(MAGMA_HAVE_HIP)
+#define cusparseZcsric0(handle, op, rows, nnz, descrA, dval, drow, dcol, info )                 \
+    {                                                                                           \
+        int bufsize;                                                                            \
+        void *buf;                                                                              \
+        csric02Info_t linfo;                                                                    \
+        hipsparseCreateCsric02Info(&linfo);                                                      \
+        hipsparseZcsric02_bufferSize(handle, rows, nnz, descrA, (hipDoubleComplex*)dval, drow, dcol,linfo,&bufsize);\
+        if (bufsize > 0)                                                                        \
+           magma_malloc(&buf, bufsize);                                                         \
+        hipsparseZcsric02_analysis(handle, rows, nnz, descrA, (hipDoubleComplex*)dval, drow, dcol, linfo,           \
+                                  HIPSPARSE_SOLVE_POLICY_NO_LEVEL, buf);                         \
+        int numerical_zero;                                                                     \
+        if (HIPSPARSE_STATUS_ZERO_PIVOT ==                                                       \
+            hipsparseXcsric02_zeroPivot( handle, linfo, &numerical_zero ))                       \
+            printf("A(%d,%d) is missing\n", numerical_zero, numerical_zero);                    \
+        hipsparseZcsric02(handle, rows, nnz, descrA, (hipDoubleComplex*)dval, drow, dcol, linfo,                    \
+                         HIPSPARSE_SOLVE_POLICY_NO_LEVEL, buf);                                  \
+        if (bufsize > 0)                                                                        \
+           magma_free(buf);                                                                     \
     } 
 #else
 #define cusparseZcsric0(handle, op, rows, nnz, descrA, dval, drow, dcol, info )                 \
@@ -151,9 +189,7 @@ magma_zcumilusetup(
     
     cusparseHandle_t cusparseHandle=NULL;
     cusparseMatDescr_t descrA=NULL;
-    cusparseMatDescr_t descrL=NULL;
-    cusparseMatDescr_t descrU=NULL;
-#if CUDA_VERSION >= 7000
+#if CUDA_VERSION >= 7000 || defined(MAGMA_HAVE_HIP)
     csrilu02Info_t info_M=NULL;
     void *pBuffer = NULL;
 #endif
@@ -185,12 +221,12 @@ magma_zcumilusetup(
     CHECK_CUSPARSE( cusparseSetMatType( descrA, CUSPARSE_MATRIX_TYPE_GENERAL ));
     CHECK_CUSPARSE( cusparseSetMatDiagType( descrA, CUSPARSE_DIAG_TYPE_NON_UNIT ));
     CHECK_CUSPARSE( cusparseSetMatIndexBase( descrA, CUSPARSE_INDEX_BASE_ZERO ));
-    cusparseCreateSolveAnalysisInfo( &(precond->cuinfo) );
+    cusparseCreateSolveAnalysisInfo( &(precond->cuinfoILU) );
 
     // use kernel to manually check for zeros n the diagonal
     CHECK( magma_zdiagcheck( precond->M, queue ) );
     
-#if CUDA_VERSION >= 7000
+#if CUDA_VERSION >= 7000 
     // this version has the bug fixed where a zero on the diagonal causes a crash
     CHECK_CUSPARSE( cusparseCreateCsrilu02Info(&info_M) );
     int buffersize;
@@ -219,19 +255,51 @@ magma_zcumilusetup(
                          precond->M.num_rows, precond->M.nnz, descrA,
                          precond->M.dval, precond->M.drow, precond->M.dcol,
                          info_M, CUSPARSE_SOLVE_POLICY_NO_LEVEL, pBuffer) );
+
+#elif defined(MAGMA_HAVE_HIP)
+
+    // this version has the bug fixed where a zero on the diagonal causes a crash
+    CHECK_CUSPARSE( hipsparseCreateCsrilu02Info(&info_M) );
+    int buffersize;
+    int structural_zero;
+    int numerical_zero;
+    
+    CHECK_CUSPARSE(
+    hipsparseZcsrilu02_bufferSize( cusparseHandle,
+                         precond->M.num_rows, precond->M.nnz, descrA,
+                         (hipDoubleComplex*)precond->M.dval, precond->M.drow, precond->M.dcol,
+                         info_M,
+                         &buffersize ) );
+    
+    CHECK( magma_malloc((void**)&pBuffer, buffersize) );
+
+    CHECK_CUSPARSE( hipsparseZcsrilu02_analysis( cusparseHandle,
+            precond->M.num_rows, precond->M.nnz, descrA,
+            (hipDoubleComplex*)precond->M.dval, precond->M.drow, precond->M.dcol,
+            info_M, CUSPARSE_SOLVE_POLICY_NO_LEVEL, pBuffer ));
+    
+    CHECK_CUSPARSE( hipsparseXcsrilu02_zeroPivot( cusparseHandle, info_M, &numerical_zero ) );
+    CHECK_CUSPARSE( hipsparseXcsrilu02_zeroPivot( cusparseHandle, info_M, &structural_zero ) );
+    
+    CHECK_CUSPARSE(
+    hipsparseZcsrilu02( cusparseHandle,
+                         precond->M.num_rows, precond->M.nnz, descrA,
+                         (hipDoubleComplex*)precond->M.dval, precond->M.drow, precond->M.dcol,
+                         info_M, HIPSPARSE_SOLVE_POLICY_NO_LEVEL, pBuffer) );
+
 #else
     // this version contains the bug but is needed for backward compability
     cusparseZcsrsm_analysis( cusparseHandle,
                              CUSPARSE_OPERATION_NON_TRANSPOSE,
                              precond->M.num_rows, precond->M.nnz, descrA,
                              precond->M.dval, precond->M.drow, precond->M.dcol,
-                             precond->cuinfo );
+                             precond->cuinfoILU );
     CHECK_CUSPARSE( cusparseZcsrilu0( cusparseHandle, CUSPARSE_OPERATION_NON_TRANSPOSE,
                       precond->M.num_rows, descrA,
                       precond->M.dval,
                       precond->M.drow,
                       precond->M.dcol,
-                      precond->cuinfo ));
+                      precond->cuinfoILU ));
 #endif
 
     CHECK( magma_zmtransfer( precond->M, &hA, Magma_DEV, Magma_CPU, queue ));
@@ -250,48 +318,49 @@ magma_zcumilusetup(
     CHECK( magma_index_malloc( &(precond->U_dgraphindegree_bak), precond->M.num_rows ));
 
     if( precond->trisolver == Magma_CUSOLVE || precond->trisolver == 0 ){
-        CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-        CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-        CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-        CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-        CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_LOWER ));
-        cusparseCreateSolveAnalysisInfo( &precond->cuinfoL );
-        cusparseZcsrsm_analysis( cusparseHandle,
-                                 CUSPARSE_OPERATION_NON_TRANSPOSE, precond->L.num_rows,
-                                 precond->L.nnz, descrL,
-                                 precond->L.dval, precond->L.drow, precond->L.dcol, 
-                                 precond->cuinfoL);
-    
-        CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-        CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-        CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-        CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-        CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_UPPER ));
-        cusparseCreateSolveAnalysisInfo( &precond->cuinfoU );
-        cusparseZcsrsm_analysis( cusparseHandle,
-                                 CUSPARSE_OPERATION_NON_TRANSPOSE, precond->U.num_rows,
-                                 precond->U.nnz, descrU,
-                                 precond->U.dval, precond->U.drow, precond->U.dcol, 
-                                 precond->cuinfoU );
+        CHECK(magma_ztrisolve_analysis(precond->L, &precond->cuinfoL, false, false, false, queue));
+        CHECK(magma_ztrisolve_analysis(precond->U, &precond->cuinfoU, true, false, false, queue));
     } else if( precond->trisolver == Magma_SYNCFREESOLVE ){
             magma_zmfree(&hL, queue );
             magma_zmfree(&hU, queue );
             magma_zmtransfer( precond->L, &hL, Magma_DEV, Magma_DEV, queue );
             // conversion using CUSPARSE
+            #ifdef MAGMA_HAVE_HIP
+            hipsparseZcsr2csc(cusparseHandle, hL.num_cols, 
+                             hL.num_rows, hL.nnz,
+                             (hipDoubleComplex*)hL.dval, hL.drow, hL.dcol, 
+                             (hipDoubleComplex*)precond->L.dval, precond->L.dcol, precond->L.drow,
+                             CUSPARSE_ACTION_NUMERIC,
+                             CUSPARSE_INDEX_BASE_ZERO);
+            #else
             cusparseZcsr2csc(cusparseHandle, hL.num_cols, 
                              hL.num_rows, hL.nnz,
                              hL.dval, hL.drow, hL.dcol, 
                              precond->L.dval, precond->L.dcol, precond->L.drow,
                              CUSPARSE_ACTION_NUMERIC,
                              CUSPARSE_INDEX_BASE_ZERO);
+
+            #endif
+
             magma_zmtransfer( precond->U, &hU, Magma_DEV, Magma_DEV, queue );
             // conversion using CUSPARSE
+
+            #ifdef MAGMA_HAVE_HIP
+            hipsparseZcsr2csc(cusparseHandle, hU.num_cols, 
+                             hU.num_rows, hU.nnz,
+                             (hipDoubleComplex*)hU.dval, hU.drow, hU.dcol, 
+                             (hipDoubleComplex*)precond->U.dval, precond->U.dcol, precond->U.drow,
+                             CUSPARSE_ACTION_NUMERIC,
+                             CUSPARSE_INDEX_BASE_ZERO);
+            #else
             cusparseZcsr2csc(cusparseHandle, hU.num_cols, 
                              hU.num_rows, hU.nnz,
                              hU.dval, hU.drow, hU.dcol, 
                              precond->U.dval, precond->U.dcol, precond->U.drow,
                              CUSPARSE_ACTION_NUMERIC,
                              CUSPARSE_INDEX_BASE_ZERO);
+            #endif
+
             // set this to be CSC
             precond->U.storage_type = Magma_CSC;
             precond->L.storage_type = Magma_CSC;
@@ -326,14 +395,12 @@ magma_zcumilusetup(
 
     
 cleanup:
-#if CUDA_VERSION >= 7000
+#if CUDA_VERSION >= 7000 || defined(MAGMA_HAVE_HIP)
     magma_free( pBuffer );
     cusparseDestroyCsrilu02Info( info_M );
 #endif
-    cusparseDestroySolveAnalysisInfo( precond->cuinfo );
+    cusparseDestroySolveAnalysisInfo( precond->cuinfoILU );
     cusparseDestroyMatDescr( descrA );
-    cusparseDestroyMatDescr( descrL );
-    cusparseDestroyMatDescr( descrU );
     cusparseDestroy( cusparseHandle );
     magma_zmfree( &hA, queue );
     magma_zmfree( &hACSR, queue );
@@ -377,13 +444,6 @@ magma_zcumilusetup_transpose(
 {
     magma_int_t info = 0;
     magma_z_matrix Ah1={Magma_CSR}, Ah2={Magma_CSR};
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrLT=NULL;
-    cusparseMatDescr_t descrUT=NULL;
-    
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
 
     // transpose the matrix
     magma_zmtransfer( precond->L, &Ah1, Magma_DEV, Magma_CPU, queue );
@@ -410,33 +470,10 @@ magma_zcumilusetup_transpose(
     magma_zmtransfer( Ah2, &(precond->UT), Magma_CPU, Magma_DEV, queue );
     magma_zmfree(&Ah2, queue );
    
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrLT ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrLT, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrLT, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrLT, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrLT, CUSPARSE_FILL_MODE_UPPER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoLT );
-    cusparseZcsrsm_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->LT.num_rows,
-                             precond->LT.nnz, descrLT,
-                             precond->LT.dval, precond->LT.drow, precond->LT.dcol, 
-                             precond->cuinfoLT );
-    
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrUT ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrUT, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrUT, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrUT, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrUT, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoUT );
-    cusparseZcsrsm_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->UT.num_rows,
-                             precond->UT.nnz, descrUT,
-                             precond->UT.dval, precond->UT.drow, precond->UT.dcol, 
-                             precond->cuinfoUT );
+    CHECK(magma_ztrisolve_analysis(precond->LT, &precond->cuinfoLT, true, false, false, queue));
+    CHECK(magma_ztrisolve_analysis(precond->UT, &precond->cuinfoUT, false, false, false, queue));
+
 cleanup:
-    cusparseDestroyMatDescr( descrLT );
-    cusparseDestroyMatDescr( descrUT );
-    cusparseDestroy( cusparseHandle );
     magma_zmfree(&Ah1, queue );
     magma_zmfree(&Ah2, queue );
 
@@ -473,10 +510,6 @@ magma_zcumilugeneratesolverinfo(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrL=NULL;
-    cusparseMatDescr_t descrU=NULL;
-    
     magma_z_matrix hA={Magma_CSR}, hL={Magma_CSR}, hU={Magma_CSR};
     
     if (precond->L.memory_location != Magma_DEV ){
@@ -495,35 +528,8 @@ magma_zcumilugeneratesolverinfo(
         magma_zmfree(&hU, queue );
     }
     
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-
-
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoL );
-    cusparseZcsrsm_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->L.num_rows,
-                             precond->L.nnz, descrL,
-                             precond->L.dval, precond->L.drow, precond->L.dcol, 
-                             precond->cuinfoL );
-
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_UPPER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoU );
-    cusparseZcsrsm_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->U.num_rows,
-                             precond->U.nnz, descrU,
-                             precond->U.dval, precond->U.drow, precond->U.dcol, 
-                             precond->cuinfoU );
-
+    CHECK(magma_ztrisolve_analysis(precond->L, &precond->cuinfoL, false, false, false, queue));
+    CHECK(magma_ztrisolve_analysis(precond->U, &precond->cuinfoU, true, false, false, queue));
     
     if( precond->trisolver != 0 && precond->trisolver != Magma_CUSOLVE ){
         //prepare for iterative solves
@@ -537,11 +543,7 @@ magma_zcumilugeneratesolverinfo(
         CHECK( magma_zvinit( &precond->work2, Magma_DEV, precond->U.num_rows, 1, MAGMA_Z_ZERO, queue ));
     }
     
-cleanup:
-    cusparseDestroyMatDescr( descrL );
-    cusparseDestroyMatDescr( descrU );
-    cusparseDestroy( cusparseHandle );
-     
+cleanup:     
     return info;
 }
 
@@ -581,36 +583,12 @@ magma_zapplycumilu_l(
     magma_queue_t queue )
 {
     magma_int_t info = 0;
-    
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrL=NULL;
-    
+        
     magmaDoubleComplex one = MAGMA_Z_MAKE( 1.0, 0.0);
 
     // CUSPARSE context //
     if( precond->trisolver == Magma_CUSOLVE || precond->trisolver == 0 ){
-        CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-        CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-        CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-        CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-        CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-        CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-        CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_LOWER ));
-        cusparseZcsrsm_solve( cusparseHandle,
-                              CUSPARSE_OPERATION_NON_TRANSPOSE,
-                              precond->L.num_rows,
-                              b.num_rows*b.num_cols/precond->L.num_rows,
-                              precond->L.nnz,
-                              &one,
-                              descrL,
-                              precond->L.dval,
-                              precond->L.drow,
-                              precond->L.dcol,
-                              precond->cuinfoL,
-                              b.dval,
-                              precond->L.num_rows,
-                              x->dval,
-                              precond->L.num_rows );
+        CHECK(magma_ztrisolve(precond->L, precond->cuinfoL, false, false, false, b, *x, queue));
     } else if( precond->trisolver == Magma_SYNCFREESOLVE ){
         magma_zgecscsyncfreetrsm_solve( precond->L.num_rows,
             precond->L.nnz, 
@@ -620,12 +598,9 @@ magma_zapplycumilu_l(
             1, // rhs
             queue );
     }
-    
-    
+       
 
 cleanup:
-    cusparseDestroyMatDescr( descrL );
-    cusparseDestroy( cusparseHandle );
     return info;
 }
 
@@ -667,40 +642,14 @@ magma_zapplycumilu_l_transpose(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrL=NULL;
-    
     magmaDoubleComplex one = MAGMA_Z_MAKE( 1.0, 0.0);
 
     // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_UPPER ));
-    cusparseZcsrsm_solve( cusparseHandle,
-                          CUSPARSE_OPERATION_NON_TRANSPOSE,
-                          precond->LT.num_rows,
-                          b.num_rows*b.num_cols/precond->LT.num_rows,
-                          precond->LT.nnz,
-                          &one,
-                          descrL,
-                          precond->LT.dval,
-                          precond->LT.drow,
-                          precond->LT.dcol,
-                          precond->cuinfoLT,
-                          b.dval,
-                          precond->LT.num_rows,
-                          x->dval,
-                          precond->LT.num_rows );
+    CHECK(magma_ztrisolve(precond->LT, precond->cuinfoLT, true, false, false, b, *x, queue));
     
     
 
 cleanup:
-    cusparseDestroyMatDescr( descrL );
-    cusparseDestroy( cusparseHandle );
     return info;
 }
 
@@ -741,35 +690,11 @@ magma_zapplycumilu_r(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrU=NULL;
-    
     magmaDoubleComplex one = MAGMA_Z_MAKE( 1.0, 0.0);
 
     // CUSPARSE context //
     if( precond->trisolver == Magma_CUSOLVE || precond->trisolver == 0 ){
-        CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-        CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-        CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-        CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-        CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-        CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-        CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_UPPER ));
-        cusparseZcsrsm_solve( cusparseHandle,
-                              CUSPARSE_OPERATION_NON_TRANSPOSE,
-                              precond->U.num_rows,
-                              b.num_rows*b.num_cols/precond->U.num_rows,
-                              precond->U.nnz,
-                              &one,
-                              descrU,
-                              precond->U.dval,
-                              precond->U.drow,
-                              precond->U.dcol,
-                              precond->cuinfoU,
-                              b.dval,
-                              precond->U.num_rows,
-                              x->dval,
-                              precond->U.num_rows );
+        CHECK(magma_ztrisolve(precond->U, precond->cuinfoU, true, false, false, b, *x, queue));
     } else if( precond->trisolver == Magma_SYNCFREESOLVE ){
         magma_zgecscsyncfreetrsm_solve( precond->U.num_rows,
             precond->U.nnz,
@@ -783,8 +708,6 @@ magma_zapplycumilu_r(
     
 
 cleanup:
-    cusparseDestroyMatDescr( descrU );
-    cusparseDestroy( cusparseHandle );
     return info; 
 }
 
@@ -825,40 +748,11 @@ magma_zapplycumilu_r_transpose(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrU=NULL;
-    
     magmaDoubleComplex one = MAGMA_Z_MAKE( 1.0, 0.0);
 
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseZcsrsm_solve( cusparseHandle,
-                          CUSPARSE_OPERATION_NON_TRANSPOSE,
-                          precond->UT.num_rows,
-                          b.num_rows*b.num_cols/precond->UT.num_rows,
-                          precond->UT.nnz,
-                          &one,
-                          descrU,
-                          precond->UT.dval,
-                          precond->UT.drow,
-                          precond->UT.dcol,
-                          precond->cuinfoUT,
-                          b.dval,
-                          precond->UT.num_rows,
-                          x->dval,
-                          precond->UT.num_rows );
-
+    CHECK(magma_ztrisolve(precond->UT, precond->cuinfoUT, false, false, false, b, *x, queue));
     
-
 cleanup:
-    cusparseDestroyMatDescr( descrU );
-    cusparseDestroy( cusparseHandle );
     return info; 
 }
 
@@ -924,11 +818,11 @@ magma_zcumiccsetup(
     CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
     CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
     CHECK_CUSPARSE( cusparseCreateMatDescr( &descrA ));
-    cusparseCreateSolveAnalysisInfo( &(precond->cuinfo) );
+    cusparseCreateSolveAnalysisInfo( &(precond->cuinfoILU) );
     // use kernel to manually check for zeros n the diagonal
     CHECK( magma_zdiagcheck( precond->M, queue ) );
- /*      
-#if CUDA_VERSION >= 7000
+    
+#if CUDA_VERSION >= 12000
     // this version has the bug fixed where a zero on the diagonal causes a crash
     CHECK_CUSPARSE( cusparseCreateCsric02Info(&info_M) );
     CHECK_CUSPARSE( cusparseSetMatType( descrA, CUSPARSE_MATRIX_TYPE_GENERAL ));
@@ -959,7 +853,7 @@ magma_zcumiccsetup(
                          precond->M.dval, precond->M.drow, precond->M.dcol,
                          info_M, CUSPARSE_SOLVE_POLICY_NO_LEVEL, pBuffer) );    
 
-#else*/
+#else
     // this version contains the bug but is needed for backward compability
     CHECK_CUSPARSE( cusparseSetMatType( descrA, CUSPARSE_MATRIX_TYPE_SYMMETRIC ));
     CHECK_CUSPARSE( cusparseSetMatDiagType( descrA, CUSPARSE_DIAG_TYPE_NON_UNIT ));
@@ -971,14 +865,14 @@ magma_zcumiccsetup(
                              CUSPARSE_OPERATION_NON_TRANSPOSE,
                              precond->M.num_rows, precond->M.nnz, descrA,
                              precond->M.dval, precond->M.drow, precond->M.dcol,
-                             precond->cuinfo );
+                             precond->cuinfoILU );
     cusparseZcsric0( cusparseHandle, CUSPARSE_OPERATION_NON_TRANSPOSE,
                      precond->M.num_rows, precond->M.nnz, descrA,
                      precond->M.dval,
                      precond->M.drow,
                      precond->M.dcol,
-                     precond->cuinfo );
-//#endif
+                     precond->cuinfoILU );
+#endif
 
     CHECK( magma_zmtransfer( precond->M, &precond->L, 
         Magma_DEV, Magma_DEV, queue ));
@@ -1035,7 +929,7 @@ cleanup:
     magma_free( pBuffer );
     cusparseDestroyCsric02Info( info_M );
 #endif
-    cusparseDestroySolveAnalysisInfo( precond->cuinfo );
+    cusparseDestroySolveAnalysisInfo( precond->cuinfoILU );
     cusparseDestroyMatDescr( descrL );
     cusparseDestroyMatDescr( descrU );
     cusparseDestroyMatDescr( descrA );
@@ -1074,36 +968,9 @@ magma_zcumicgeneratesolverinfo(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrL=NULL;
-    cusparseMatDescr_t descrU=NULL;
+    CHECK(magma_ztrisolve_analysis(precond->M, &precond->cuinfoL, false, false, false, queue));
+    CHECK(magma_ztrisolve_analysis(precond->M, &precond->cuinfoU, false, false, true, queue));
     
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoL );
-    cusparseZcsrsm_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->M.num_rows,
-                             precond->M.nnz, descrL,
-                             precond->M.dval, precond->M.drow, precond->M.dcol, 
-                             precond->cuinfoL );
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoU );
-    cusparseZcsrsm_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_TRANSPOSE, precond->M.num_rows,
-                             precond->M.nnz, descrU,
-                             precond->M.dval, precond->M.drow, precond->M.dcol, 
-                             precond->cuinfoU );
-
 
 /*
     // to enable also the block-asynchronous iteration for the triangular solves
@@ -1132,9 +999,6 @@ magma_zcumicgeneratesolverinfo(
 */
 
 cleanup:
-    cusparseDestroyMatDescr( descrL );
-    cusparseDestroyMatDescr( descrU );
-    cusparseDestroy( cusparseHandle );
     return info;
 }
 
@@ -1176,40 +1040,11 @@ magma_zapplycumicc_l(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrL=NULL;
-    
     magmaDoubleComplex one = MAGMA_Z_MAKE( 1.0, 0.0);
 
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_LOWER ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-    cusparseZcsrsm_solve( cusparseHandle,
-                          CUSPARSE_OPERATION_NON_TRANSPOSE,
-                          precond->M.num_rows,
-                          b.num_rows*b.num_cols/precond->M.num_rows,
-                          precond->M.nnz,
-                          &one,
-                          descrL,
-                          precond->M.dval,
-                          precond->M.drow,
-                          precond->M.dcol,
-                          precond->cuinfoL,
-                          b.dval,
-                          precond->M.num_rows,
-                          x->dval,
-                          precond->M.num_rows );
-
-    
+    CHECK(magma_ztrisolve(precond->M, precond->cuinfoL, false, false, false, b, *x, queue));
 
 cleanup:
-    cusparseDestroyMatDescr( descrL );
-    cusparseDestroy( cusparseHandle );
     return info; 
 }
 
@@ -1250,40 +1085,13 @@ magma_zapplycumicc_r(
 {
     magma_int_t info = 0;
     
-    cusparseHandle_t cusparseHandle=NULL;
-    cusparseMatDescr_t descrU=NULL;
-    
     magmaDoubleComplex one = MAGMA_Z_MAKE( 1.0, 0.0);
 
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseSetStream( cusparseHandle, queue->cuda_stream() ));
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseZcsrsm_solve( cusparseHandle,
-                          CUSPARSE_OPERATION_TRANSPOSE,
-                          precond->M.num_rows,
-                          b.num_rows*b.num_cols/precond->M.num_rows,
-                          precond->M.nnz,
-                          &one,
-                          descrU,
-                          precond->M.dval,
-                          precond->M.drow,
-                          precond->M.dcol,
-                          precond->cuinfoU,
-                          b.dval,
-                          precond->M.num_rows,
-                          x->dval,
-                          precond->M.num_rows );
+    CHECK(magma_ztrisolve(precond->M, precond->cuinfoU, false, false, true, b, *x, queue));
     
     
 
 cleanup:
-    cusparseDestroyMatDescr( descrU );
-    cusparseDestroy( cusparseHandle );
     return info; 
 }
 
