@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @precisions normal z -> s d c
        @author Hartwig Anzt
@@ -12,6 +12,13 @@
 
 #include <cuda.h>  // for CUDA_VERSION
 
+
+/* For hipSPARSE, they use a separate complex type than for hipBLAS */
+#ifdef MAGMA_HAVE_HIP
+  #define hipblasDoubleComplex hipDoubleComplex
+#endif
+
+
 // todo: check if we need buf later
 #if CUDA_VERSION >= 11000
 #define cusparseZcsr2csc(handle, m, n, nnz, valA, rowA, colA, valB, rowB, colB,                \
@@ -23,7 +30,7 @@
                                       valB, rowB, colB, CUDA_C_64F, action, base,              \
                                       CUSPARSE_CSR2CSC_ALG1, &bufsize);                        \
         if (bufsize > 0)                                                                       \
-           magma_malloc(&buf, bufsize);                                                        \
+           CHECK( magma_malloc(&buf, bufsize) );                                                        \
         cusparseCsr2cscEx2(handle, m, n, nnz, valA, rowA, colA, valB, rowB, colB,              \
                            CUDA_C_64F, action, base, CUSPARSE_CSR2CSC_ALG1, buf);              \
         if (bufsize > 0)                                                                       \
@@ -1750,9 +1757,9 @@ magma_zmconvert(
             // conversion using CUSPARSE
             cusparseZcsr2bsr( cusparseHandle, CUSPARSE_DIRECTION_ROW,
                               A.num_rows, A.num_cols, descr,
-                              A.dval, A.drow, A.dcol,
+                              (cuDoubleComplex*)A.dval, A.drow, A.dcol,
                               size_b, descr,
-                              B->dval, B->drow, B->dcol);
+                              (cuDoubleComplex*)B->dval, B->drow, B->dcol);
         }
         // BCSR to CSR
         else if ( old_format == Magma_BCSR && new_format == Magma_CSR ) {
@@ -1784,9 +1791,9 @@ magma_zmconvert(
 
             // conversion using CUSPARSE
             cusparseZbsr2csr( cusparseHandle, CUSPARSE_DIRECTION_ROW,
-                              mb, nb, descr, A.dval, A.drow, A.dcol,
+                              mb, nb, descr, (cuDoubleComplex*)A.dval, A.drow, A.dcol,
                               size_b, descr,
-                              B->dval, B->drow, B->dcol );
+                              (cuDoubleComplex*)B->dval, B->drow, B->dcol );
         }
         // CSR to CSC
         else if ( old_format == Magma_CSR && new_format == Magma_CSC ) {
@@ -1812,8 +1819,8 @@ magma_zmconvert(
 
             // conversion using CUSPARSE
             cusparseZcsr2csc(cusparseHandle, A.num_rows, A.num_cols, A.nnz,
-                             A.dval, A.drow, A.dcol,
-                             B->dval, B->drow, B->dcol,
+                             (cuDoubleComplex*)A.dval, A.drow, A.dcol,
+                             (cuDoubleComplex*)B->dval, B->drow, B->dcol,
                              CUSPARSE_ACTION_NUMERIC,
                              CUSPARSE_INDEX_BASE_ZERO);
         }
@@ -1841,8 +1848,8 @@ magma_zmconvert(
 
             // conversion using CUSPARSE
             cusparseZcsr2csc(cusparseHandle, A.num_cols, A.num_rows, A.nnz,
-                             A.dval, A.dcol, A.drow,
-                             B->dval, B->dcol, B->drow,
+                             (cuDoubleComplex*)A.dval, A.dcol, A.drow,
+                             (cuDoubleComplex*)B->dval, B->dcol, B->drow,
                              CUSPARSE_ACTION_NUMERIC,
                              CUSPARSE_INDEX_BASE_ZERO);
         }
@@ -1913,19 +1920,31 @@ magma_zmconvert(
 
             // step 1: allocate buffer
             cusparseXcoosort_bufferSizeExt(cusparseHandle, m, n, nnz, B->drowidx, B->dcol, &pBufferSizeInBytes);
-            //cudaMalloc( &pBuffer, sizeof(char)* pBufferSizeInBytes);
             CHECK( magma_malloc( &pBuffer, sizeof(char)* pBufferSizeInBytes ));
             // step 2: setup permutation vector P to identity
             CHECK( magma_index_malloc( &P, nnz ));
-            //magma_( &P, sizeof(int)*nnz);
             cusparseCreateIdentityPermutation(cusparseHandle, nnz, P);
 
             // step 3: sort COO format by Row
             cusparseXcoosortByRow(cusparseHandle, m, n, nnz, B->drowidx, B->dcol, P, pBuffer);
 
             // step 4: gather sorted cooVals
+#if CUDA_VERSION >= 12000
+            cusparseSpVecDescr_t vec_permutation;
+            cusparseDnVecDescr_t vec_values;
+            CHECK_CUSPARSE( cusparseCreateSpVec(&vec_permutation, A.nnz, A.nnz,
+                                                P, B->dval,
+                                                CUSPARSE_INDEX_32I,
+                                                CUSPARSE_INDEX_BASE_ZERO, CUDA_C_64F) );
+            CHECK_CUSPARSE( cusparseCreateDnVec(&vec_values, A.nnz, A.dval, CUDA_C_64F) );
+            CHECK_CUSPARSE( cusparseGather(cusparseHandle, vec_values, vec_permutation) );
+            
+            // destroy matrix/vector descriptors
+            CHECK_CUSPARSE( cusparseDestroySpVec(vec_permutation) );
+            CHECK_CUSPARSE( cusparseDestroyDnVec(vec_values) );
+#else
             cusparseZgthr(cusparseHandle, nnz, A.dval, B->dval, P, CUSPARSE_INDEX_BASE_ZERO);
-
+#endif
 
             // conversion using CUSPARSE
             cusparseXcoo2csr( cusparseHandle, B->drowidx,
