@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @precisions normal z -> c d s
        @author Hartwig Anzt
@@ -197,7 +197,12 @@ magma_z_precondsetup(
     
     else if ( precond->solver == Magma_PARILUT ) {
         #ifdef _OPENMP
+        /* Here, use No-Dynamic-Parallelism vresion or normal version depending on platform */
+            #ifdef MAGMA_HAVE_HIP
+            info = magma_zparilut_gpu_nodp(A, b, precond, queue);
+            #else
             info = magma_zparilut_gpu( A, b, precond, queue );
+            #endif
             if ( precond->trisolver == Magma_ISAI  ||
                 precond->trisolver == Magma_JACOBI ||
                 precond->trisolver == Magma_VBJACOBI ){
