@@ -1,10 +1,10 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
 
-       @date October 2020
+       @date August 2023
             
        @author Stan Tomov
        @author Hartwig Anzt
