@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Hartwig Anzt
 
@@ -15,33 +15,14 @@
 #include <omp.h>
 #endif
 
+
+#include "../blas/magma_trisolve.h"
+
 #define PRECISION_z
 
-// todo: make it spacific
-#if CUDA_VERSION >= 11000
-#define cusparseCreateSolveAnalysisInfo(info) {;}
-#else
-#define cusparseCreateSolveAnalysisInfo(info)                                                   \
-    CHECK_CUSPARSE( cusparseCreateSolveAnalysisInfo( info ))
-#endif
-
-// todo: info is passed; buf has to be passed 
-#if CUDA_VERSION >= 11000
-#define cusparseZcsrsv_analysis(handle, trans, m, nnz, descr, val, row, col, info)              \
-    {                                                                                           \
-        csrsv2Info_t linfo = 0;                                                                 \
-        int bufsize;                                                                            \
-        void *buf;                                                                              \
-        cusparseCreateCsrsv2Info(&linfo);                                                       \
-        cusparseZcsrsv2_bufferSize(handle, trans, m, nnz, descr, val, row, col,                 \
-                                   linfo, &bufsize);                                            \
-        if (bufsize > 0)                                                                        \
-           magma_malloc(&buf, bufsize);                                                         \
-        cusparseZcsrsv2_analysis(handle, trans, m, nnz, descr, val, row, col, linfo,            \
-                                 CUSPARSE_SOLVE_POLICY_USE_LEVEL, buf);                         \
-        if (bufsize > 0)                                                                        \
-           magma_free(buf);                                                                     \
-    }
+/* For hipSPARSE, they use a separate complex type than for hipBLAS */
+#ifdef MAGMA_HAVE_HIP
+  #define hipblasDoubleComplex hipDoubleComplex
 #endif
 
 /***************************************************************************//**
@@ -453,31 +434,9 @@ magma_zparilut(
     //magma_zmtranspose(U, &UT, queue );
     CHECK( magma_zmtransfer( UT, &precond->U, Magma_CPU, Magma_DEV , queue ));
 
-    // CUSPARSE context //
-    CHECK_CUSPARSE( cusparseCreate( &cusparseHandle ));
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrL ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrL, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrL, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrL, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrL, CUSPARSE_FILL_MODE_LOWER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoL );
-    cusparseZcsrsv_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->L.num_rows,
-                             precond->L.nnz, descrL,
-                             precond->L.dval, precond->L.drow, precond->L.dcol, 
-                             precond->cuinfoL );
-    CHECK_CUSPARSE( cusparseCreateMatDescr( &descrU ));
-    CHECK_CUSPARSE( cusparseSetMatType( descrU, CUSPARSE_MATRIX_TYPE_TRIANGULAR ));
-    CHECK_CUSPARSE( cusparseSetMatDiagType( descrU, CUSPARSE_DIAG_TYPE_NON_UNIT ));
-    CHECK_CUSPARSE( cusparseSetMatIndexBase( descrU, CUSPARSE_INDEX_BASE_ZERO ));
-    CHECK_CUSPARSE( cusparseSetMatFillMode( descrU, CUSPARSE_FILL_MODE_UPPER ));
-    cusparseCreateSolveAnalysisInfo( &precond->cuinfoU );
-    cusparseZcsrsv_analysis( cusparseHandle,
-                             CUSPARSE_OPERATION_NON_TRANSPOSE, precond->U.num_rows,
-                             precond->U.nnz, descrU,
-                             precond->U.dval, precond->U.drow, precond->U.dcol, 
-                             precond->cuinfoU );
-
+    CHECK(magma_ztrisolve_analysis(precond->L, &precond->cuinfoL, false, false, false, queue));
+    CHECK(magma_ztrisolve_analysis(precond->U, &precond->cuinfoU, true, false, false, queue));
+    
     if( precond->trisolver != 0 && precond->trisolver != Magma_CUSOLVE ){
         //prepare for iterative solves
 
