@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Azzam Haidar
        @author Stan Tomov
@@ -11,14 +11,13 @@
   
        @precisions normal z -> s d c
 */
-#include <cuda_runtime.h>
 
 #include "magma_internal.h"
 #include "magma_bulge.h"
 #include "magma_zbulgeinc.h"
 
 // =============================================================================
-#ifdef HAVE_clBLAS
+#ifdef MAGMA_HAVE_OPENCL
 #define dE(i,j)  (dE, (dE_offset + (i) + ldde*(j)))
 #else
 #define dE(i,j)  (dE + (i) + ldde*(j))
@@ -98,10 +97,10 @@ magma_zbulge_applyQ_v2(
     magma_queue_create( cdev, &queues[1] );
 
     magma_event_t myevent[2];
-    cudaEventCreateWithFlags(&myevent[0],cudaEventDisableTiming);
-    cudaEventCreateWithFlags(&myevent[1],cudaEventDisableTiming);
-    //magma_event_create(&myevent[0]);
-    //magma_event_create(&myevent[1]);
+    //cudaEventCreateWithFlags(&myevent[0],cudaEventDisableTiming);
+    //cudaEventCreateWithFlags(&myevent[1],cudaEventDisableTiming);
+    magma_event_create_untimed(&myevent[0]);
+    magma_event_create_untimed(&myevent[1]);
 
 
     // Azzam 21/11/2012
