@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Azzam Haidar
        @author Stan Tomov
@@ -11,7 +11,6 @@
        @precisions normal z -> s d c
 
 */
-#include <cuda_runtime.h>
 
 #include "magma_internal.h"
 #include "trace.h"
@@ -151,7 +150,7 @@ magma_zhetrd_he2hb(
     magmaDoubleComplex_ptr dT,
     magma_int_t *info)
 {
-    #ifdef HAVE_clBLAS
+    #ifdef MAGMA_HAVE_OPENCL
     #define dA(a_1,a_2)  (dA, (dA_offset + ((a_2)-1)*(ldda) + (a_1)-1))
     #define dT(a_1)      (dT, (dT_offset + ((a_1)-1)*(lddt)))
     #else
@@ -237,8 +236,8 @@ magma_zhetrd_he2hb(
     memset( hT, 0, nb*nb*sizeof(magmaDoubleComplex));
 
     magma_event_t Pupdate_event;
-    cudaEventCreateWithFlags(&Pupdate_event,cudaEventDisableTiming);
-    //magma_event_create(&Pupdate_event);
+    //cudaEventCreateWithFlags(&Pupdate_event,cudaEventDisableTiming);
+    magma_event_create_untimed(&Pupdate_event);
 
 
     if (upper) {
