@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Azzam Haidar
        @author Stan Tomov
@@ -11,7 +11,6 @@
        @precisions normal z -> s d c
 
 */
-#include <cuda_runtime.h>
 
 #include "magma_internal.h"
 #include "magma_bulge.h"
@@ -186,7 +185,7 @@ magma_zhetrd_he2hb_mgpu(
     magma_queue_t queues[][20], magma_int_t nqueue,
     magma_int_t *info)
 {
-    #ifdef HAVE_clBLAS
+    #ifdef MAGMA_HAVE_OPENCL
     #define dT(a_0, a_1, a_2) (dTmgpu[a_0], (dTmgpu_offset + ((a_2)-1)*(lddt) + (a_1)-1)
     #define dA(a_0, a_1, a_2) (dAmgpu[a_0], (dAmgpu_offset + ((a_2)-1)*(ldda) + (a_1)-1)
     #else
@@ -278,8 +277,8 @@ magma_zhetrd_he2hb_mgpu(
         dwork[dev]    = dw[dev]      + nb*lddw;
         dworkbis[dev] = dwork[dev]   + nb*ldda;
         for( i = 0; i < nevents; ++i ) {
-            cudaEventCreateWithFlags( &events[dev][i], cudaEventDisableTiming );
-            //magma_create_event( &events[dev][i] );
+            //cudaEventCreateWithFlags( &events[dev][i], cudaEventDisableTiming );
+            magma_event_create_untimed( &events[dev][i] );
         }
     }
 
