@@ -1,15 +1,13 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @precisions normal z -> s d c
 
 */
-#include <cuda_runtime.h>
-
 #include "magma_internal.h"
 
 //#include "../testing/flops.h"
@@ -137,7 +135,7 @@ magma_zpotrf_m(
 
     /* figure out NB */
     size_t freeMem, totalMem;
-    cudaMemGetInfo( &freeMem, &totalMem );
+    magma_mem_info(&freeMem, &totalMem);
     freeMem /= sizeof(magmaDoubleComplex);
     
     //MB = n;  /* number of rows in the big panel    */
