@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Mark Gates
 
@@ -25,15 +25,18 @@
 
 // --------------------
 // global variable
-#if   defined(HAVE_CUBLAS)
+#if   defined(MAGMA_HAVE_CUDA)
     const char* g_platform_str = "cuBLAS";
 
-#elif defined(HAVE_clBLAS)
+#elif defined(MAGMA_HAVE_OPENCL)
     const char* g_platform_str = "clBLAS";
 
 #elif defined(HAVE_MIC)
     const char* g_platform_str = "Xeon Phi";
 
+#elif defined(MAGMA_HAVE_HIP)
+    const char* g_platform_str = "HIPBLAS";
+
 #else
     #error "unknown platform"
 #endif
@@ -50,6 +53,7 @@ void magma_assert( bool condition, const char* msg, ... )
         va_list va;
         va_start( va, msg );
         vprintf( msg, va );
+        va_end( va );
         printf( "\n" );
         exit(1);
     }
@@ -66,6 +70,7 @@ void magma_assert_warn( bool condition, const char* msg, ... )
         va_list va;
         va_start( va, msg );
         vprintf( msg, va );
+        va_end( va );
         printf( "\n" );
     }
 }
@@ -629,7 +634,7 @@ void magma_opts::parse_opts( int argc, char** argv )
     }
     assert( this->ntest <= MAX_NTEST );
 
-    #if defined(HAVE_CUBLAS) || defined(HAVE_HIP)
+    #if defined(MAGMA_HAVE_CUDA) || defined(MAGMA_HAVE_HIP)
     magma_setdevice( this->device );
     #endif
 
@@ -641,10 +646,10 @@ void magma_opts::parse_opts( int argc, char** argv )
 
     this->queue = this->queues2[ 0 ];
 
-    #if defined(HAVE_HIP)
+    #if defined(MAGMA_HAVE_HIP)
         // handle for directly calling hipblas
         this->handle = magma_queue_get_hipblas_handle( this->queue );
-    #elif defined(HAVE_CUBLAS)
+    #elif defined(MAGMA_HAVE_CUDA)
         // handle for directly calling cublas
         this->handle = magma_queue_get_cublas_handle( this->queue );
     #else
@@ -710,7 +715,7 @@ void magma_opts::cleanup()
     this->queues2[0] = NULL;
     this->queues2[1] = NULL;
 
-    #if defined(HAVE_CUBLAS) || defined(HAVE_HIP)
+    #if defined(MAGMA_HAVE_CUDA) || defined(MAGMA_HAVE_HIP)
     this->handle = NULL;
     #endif
 }
