@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @author Mark Gates
 */
@@ -424,7 +424,7 @@ int main( int argc, char** argv )
     printf( "LAPACK -> magma_xxxxx_const     %s\n", (s == gStatus ? "ok" : "failed"));
 
     // ------------------------------------------------------------
-    #ifdef HAVE_clBLAS
+    #ifdef MAGMA_HAVE_OPENCL
     s = gStatus;
     check( clblas_order_const( MagmaRowMajor      ) == clblasRowMajor    );
     check( clblas_order_const( MagmaColMajor      ) == clblasColumnMajor );
