@@ -1,9 +1,9 @@
 /*
-    -- MAGMA (version 2.5.4) --
+    -- MAGMA (version 2.7.2) --
        Univ. of Tennessee, Knoxville
        Univ. of California, Berkeley
        Univ. of Colorado, Denver
-       @date October 2020
+       @date August 2023
 
        @precisions normal z -> c d s
        @author Chongxiao Cao
@@ -80,7 +80,7 @@ int main( int argc, char** argv)
     printf("%% side = %s, uplo = %s, transA = %s, diag = %s \n",
            lapack_side_const(opts.side), lapack_uplo_const(opts.uplo),
            lapack_trans_const(opts.transA), lapack_diag_const(opts.diag) );
-    printf("%% BatchCount   M     N   MAGMA Gflop/s (ms)   CUBLAS Gflop/s (ms)    CPU Gflop/s (ms)      MAGMA     CUBLAS   LAPACK error\n");
+    printf("%% BatchCount   M     N   MAGMA Gflop/s (ms)   %s Gflop/s (ms)    CPU Gflop/s (ms)      MAGMA     %s   LAPACK error\n", g_platform_str, g_platform_str);
     printf("%%=========================================================================================================================\n");
     for( int itest = 0; itest < opts.ntest; ++itest ) {
         for( int iter = 0; iter < opts.niter; ++iter ) {
@@ -207,12 +207,22 @@ int main( int argc, char** argv)
             magma_zset_pointer( d_B_array, d_B, lddb, 0, 0, lddb*N, batchCount, opts.queue );
 
             cublas_time = magma_sync_wtime( opts.queue );
+            #ifdef MAGMA_HAVE_CUDA
             cublasZtrsmBatched(
                 opts.handle, cublas_side_const(opts.side), cublas_uplo_const(opts.uplo),
                 cublas_trans_const(opts.transA), cublas_diag_const(opts.diag),
                 int(M), int(N), (const cuDoubleComplex*)&alpha,
-                (cuDoubleComplex* const*) d_A_array, int(ldda),
+                (const cuDoubleComplex**) d_A_array, int(ldda),
                 (      cuDoubleComplex**) d_B_array, int(lddb), int(batchCount) );
+            #else
+            hipblasZtrsmBatched(
+                opts.handle, cublas_side_const(opts.side), cublas_uplo_const(opts.uplo),
+                cublas_trans_const(opts.transA), cublas_diag_const(opts.diag),
+                int(M), int(N), (const hipblasDoubleComplex*)&alpha,
+                (hipblasDoubleComplex* const*) d_A_array, int(ldda),
+                (      hipblasDoubleComplex**) d_B_array, int(lddb), int(batchCount) );
+            #endif
+
             cublas_time = magma_sync_wtime( opts.queue ) - cublas_time;
             cublas_perf = gflops / cublas_time;
 
